function criePosicoes(valores) {
    var r = [];
    for (var i = 0; i < valores.length; i += 2) {
        r.push(new Posicao(valores[i], valores[i+1]));
    }
    return r
}

CampoMinadoTest = TestCase("CampoMinadoTest",{
    testEncontreAsPosicoesAoRedor: function() {
        var campo = new CampoMinado(3, 3);

        function test(linha, coluna, posicoesAoRedor) {
            var esperado = criePosicoes(posicoesAoRedor);
            var atual = campo.encontreAsPosicoesAoRedor(new Posicao(linha, coluna));
            assertEquals(esperado, atual);
        }

        // cada dois números no array representam uma posição ao redor esperada
        test(0, 0, [0, 1, 1, 0, 1, 1]); // as posições ao redor de (0,0) são (0,1) (1,0) (1,1)
        test(0, 2, [0, 1, 1, 1, 1, 2]);
        test(2, 0, [2, 1, 1, 1, 1, 0]);
        test(2, 2, [1, 2, 1, 1, 2, 1]);
        test(1, 1, [1, 2, 0, 2, 0, 1, 0, 0, 1, 0, 2, 0, 2, 1, 2, 2]);
    },

    testAdicioneBomba: function () {
        var campo = new CampoMinado(3, 3);

        function test(linha, coluna, esperado) {
            var p = new Posicao(linha, coluna);
            campo.adicioneBomba(p);
            assertEquals(esperado, campo.numBombas);
            assertTrue(campo.quadrado(p).temBomba)
        }

        test(0, 0, 1); // adiciona uma bomba em (0,0) e espera que o total de bombas seja 1
        test(0, 2, 2);
        test(1, 1, 3);
        test(2, 2, 4);
        test(2, 2, 4); // adicionar em um quadrado que já tem bomba não deve ter efeito

	// cada valor representa o número de bombas ao redor. Ex: no quadrado
	// (1,0) tem que ter duas bombas ao redor, a do quadrado (0,0) e (1,1)

        var esperado = [[1, 3, 1],
                        [2, 3, 3],
                        [1, 2, 1]];
        // verifica se o número de bombas ao redor foi calculado corretamente
        for (var linha = 0; linha < 3; linha += 1) {
            for (var coluna = 0; coluna < 3; coluna += 1) {
                var p = new Posicao(linha, coluna);
                var atual = campo.quadrado(p).numBombasAoRedor;
                assertEquals("Posicao " + linha + " " + coluna, esperado[linha][coluna], atual);
            }
        }
    },

    testCrieBombas: function () {
        var campo = new CampoMinado(3, 3);

        campo.crieBombas(8);
        assertEquals(8, campo.numBombas);

        // conta as bombas no campo para verificar se a quantidade criada está correta
        var numBombas = 0;
        for (var linha = 0; linha < 3; linha += 1) {
            for (var coluna = 0; coluna < 3; coluna += 1) {
                var p = new Posicao(linha, coluna);
                if (campo.quadrado(p).temBomba) {
                    numBombas += 1;
                }
            }
        }
        assertEquals(8, numBombas);

        // tenta adicionar mais bombas que quadrados no campo
        try {
            campo.crieBombas(2);
            fail("já existem 8 bombas, como tem 9 quadrados no campo, não é possível criar mais 2 bombas");
        } catch (e) {      
        }
    },
    
    testAbraQuadrado: function () {
        var campo = new CampoMinado(4, 4);

        campo.adicioneBomba(new Posicao(0, 0));
        campo.adicioneBomba(new Posicao(0, 2));
        campo.adicioneBomba(new Posicao(1, 0));

        campo.abraQuadrado(new Posicao(2, 2));

        // os valores 1 são os quadrados que devem estar aberto
        var esperado = [[0, 0, 0, 0],
                        [0, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]];
        for (var linha = 0; linha < 4; linha += 1) {
            for (var coluna = 0; coluna < 4; coluna += 1) {
                var p = new Posicao(linha, coluna);
                var atual = campo.quadrado(p).estaAberto;
                assertEquals("Posicao " + linha + " " + coluna, Boolean(esperado[linha][coluna]), atual);
            }
        }
    },

    testAbraQuadradoComBandeira: function () {
        var campo = new CampoMinado(2, 2);
        campo.adicioneBomba(new Posicao(0, 0));

        // não deve ser possível abrir um quadrado com bandeira
        campo.mudeBandeira(new Posicao(1, 1));
        campo.abraQuadrado(new Posicao(1, 1));
        assertFalse(campo.quadrado(new Posicao(1, 1)).estaAberto);
    },

    testMudeBandeira: function () {
        var campo = new CampoMinado(2, 2);

        campo.adicioneBomba(new Posicao(0, 0));
        campo.adicioneBomba(new Posicao(1, 1));

        // coloca uma bandeira
        campo.mudeBandeira(new Posicao(0, 0));
        assertEquals(1, campo.numBandeiras);
        assertTrue(campo.quadrado(new Posicao(0, 0)).temBandeira);

        // retira uma bandeira
        campo.mudeBandeira(new Posicao(0, 0));
        assertEquals(0, campo.numBandeiras);
        assertFalse(campo.quadrado(new Posicao(0, 0)).temBandeira);

        // tenta colocar uma bandeira em um quadrado aberto
        campo.abraQuadrado(new Posicao(0, 1));
        campo.mudeBandeira(new Posicao(0, 1));
        assertEquals(0, campo.numBandeiras);
        assertFalse(campo.quadrado(new Posicao(0, 1)).temBandeira);
    },

    testJogoAcabouEJogadorGanhou: function() {
        // cria um campo minado com 4 quadrado e duas bombas
        var campo = new CampoMinado(2, 2);
        campo.adicioneBomba(new Posicao(0, 0));
        campo.adicioneBomba(new Posicao(1, 1));
        assertFalse(campo.jogoAcabou);
        assertFalse(campo.jogadorGanhou);

        // abre um quadrado sem bomba
        campo.abraQuadrado(new Posicao(0, 1));
        assertFalse(campo.jogoAcabou);
        assertFalse(campo.jogadorGanhou);

        // abre o outro quadrado sem bomba
        campo.abraQuadrado(new Posicao(1, 0));
        assertTrue(campo.jogoAcabou);
        assertTrue(campo.jogadorGanhou);
    },

    testJogoAcabouEJogadorPerdeu: function() {
        //cria um campo minado com 4 quadrados e duas bombas
        var campo = new CampoMinado(2, 2);
        campo.adicioneBomba(new Posicao(0, 0));
        campo.adicioneBomba(new Posicao(1, 1));
        assertFalse(campo.jogoAcabou);
        assertFalse(campo.jogadorGanhou);

        // abre um quadrado sem bomba
        campo.abraQuadrado(new Posicao(0, 1));
        assertFalse(campo.jogoAcabou);
        assertFalse(campo.jogadorGanhou);

        // abre um quadrado com bomba
        campo.abraQuadrado(new Posicao(0, 0));
        assertTrue(campo.jogoAcabou);
        assertFalse(campo.jogadorGanhou);
    }
});
