function criarCampoPadrao(id) {
    return criarCampo(8, 8, 10, id, 'campo-minado.png');
}

function criarCampo(altura, largura, bombas, id, imagemSrc) {
    var imagem = new Image();
    imagem.onload = function () {
       var campo = new CampoMinado(altura, largura);
       campo.crieBombas(bombas);
       var campoui = new CampoMinadoUi(campo, id, imagem);
    }
    imagem.src = imagemSrc;
}

function CampoMinadoUi(campo, id, imagem) {
    this.campo = campo;

    //indices para as imagens dos quadrados
    this.indices = {};
    this.indices.fechado = 0;
    this.indices.fechadoBomba = 1;
    for (var i = 0; i <= 8; i += 1) {
        this.indices[i] = 3 + i;
    }
    this.indices.abertoBomba = 2;
    this.indices.bandeiraErrada = 12;
    this.indices.bandeira = 13;

    this.imagem = imagem;
    this.tamanhoQuadrado = this.imagem.height;

    // outro nome para this poder ser usado nas closure
    var ui = this;

    // criar um link que server para reinicar o jogo
    var novo = document.createElement("a");
    novo.textContent = "Novo";
    novo.href = "javascript:";
    novo.addEventListener('click', function(e) {ui.reiniciar()}, false);

    // criar o canvas
    this.canvas = document.createElement("canvas");
    this.canvas.oncontextmenu = function () {return false}; //habilita o clique do mouse com botão direito
    this.canvas.width = this.tamanhoQuadrado * this.campo.largura;
    this.canvas.height = this.tamanhoQuadrado * this.campo.altura;
    this.canvas.addEventListener("mouseup", function (e){ui.trataClique(e)}, false);
    this.context = this.canvas.getContext("2d");

    // criar uma barra de informação
    this.info = document.createElement('div');
   
    // adiciona os elementos criados ao elemento onde ficará o jogo
    var alvo = document.getElementById(id);
    alvo.appendChild(novo);
    alvo.appendChild(document.createElement('br'));
    alvo.appendChild(this.canvas);
    alvo.appendChild(this.info);

    // timer que conta o tempo do jogo
    this.timer = new Timer();
    setInterval(function () {ui.atualizaInfo()}, 1000);

    this.desenheCampo();
}

CampoMinadoUi.prototype = {
    desenheCampo: function () {
        for (var linha = 0; linha < this.campo.altura; linha += 1) {
            for (var coluna = 0; coluna < this.campo.largura; coluna += 1) {
                var p = new Posicao(linha, coluna);
                this.desenheQuadrado(p)
            }
        }
        this.atualizaInfo();
    },

    atualizaInfo: function() {
        var texto = '<p>Bandeiras: ' + this.campo.numBandeiras + '/' + this.campo.numBombas +
                    ' | Tempo: ' + this.timer.tempoHMS() + '</p>';

        if (this.campo.jogoAcabou) {
           if (this.campo.jogadorGanhou) {
             texto += '<p>Você ganhou, parabéns!</p>';
           } else {
             texto += '<p>Você perdeu, tente novamente!</p>';
           }
        }
        this.info.innerHTML = texto;
    },

    desenheQuadrado: function (p) {
        var q = this.campo.quadrado(p);
        var indice;
        if (q.estaAberto) {
            if (q.temBomba) {
                indice = this.indices['abertoBomba'];
            } else {
                indice = this.indices[q.numBombasAoRedor];
            }
        } else {
            if (q.temBandeira) {
                if (! q.temBomba && this.campo.jogoAcabou) {
                    indice = this.indices['bandeiraErrada'];
                } else {
                    indice = this.indices['bandeira'];
                }
            } else {
                if (q.temBomba && this.campo.jogoAcabou) {
                    indice = this.indices['fechadoBomba'];
                } else {
                    indice = this.indices['fechado'];
                }
            }
        }

        this.desenheSubImagem(p, indice);
    },

    desenheSubImagem: function (p, indice) {
        this.context.drawImage(this.imagem,
            this.tamanhoQuadrado * indice, 0, this.tamanhoQuadrado, this.tamanhoQuadrado,
            this.tamanhoQuadrado * p.coluna, this.tamanhoQuadrado * p.linha,
            this.tamanhoQuadrado, this.tamanhoQuadrado);
    },

    getPosicaoClique: function (e) {
        var x;
        var y;
        if (e.pageX != undefined && e.pageY != undefined) {
            x = e.pageX;
            y = e.pageY;
        } else {
            x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
            y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
        }

        x -= this.canvas.offsetLeft;
        y -= this.canvas.offsetTop;

        var linha = Math.floor(y/this.tamanhoQuadrado);
        var coluna = Math.floor(x/this.tamanhoQuadrado);

        return new Posicao(linha, coluna);
    },

    trataClique: function (e) {
        if (this.campo.jogoAcabou) {
            return;
        }

        if (this.timer.parado) {
            this.timer.inicia();
        }
        
        var p = this.getPosicaoClique(e);
        if (e.button == 0) {        // botão esquerdo
            this.campo.abraQuadrado(p);
        } else if (e.button == 2) { // botão direito
            this.campo.mudeBandeira(p);
        }

        if (this.campo.jogoAcabou) {
            this.timer.para();
        }

        this.desenheCampo();
    },

    reiniciar: function() {
       this.timer.para();
       this.timer.zera();

       var old = this.campo;
       this.campo = new CampoMinado(old.altura, old.largura);
       this.campo.crieBombas(old.numBombas);

       this.desenheCampo();
    }
}
