function Posicao(linha, coluna) {
    this.linha = linha;
    this.coluna = coluna;
}

function Quadrado() {
    this.estaAberto = false;
    this.temBomba = false;
    this.temBandeira = false;
    this.numBombasAoRedor = 0;
}

function CampoMinado(altura, largura) {
    this.altura = altura;
    this.largura = largura;
    this.numQuadrados = this.altura * this.largura;
    this.numQuadradosAbertos = 0;
    this.numBombas = 0;
    this.numBandeiras = 0;
    this.quadrados = matriz(Quadrado, altura, largura);
    this.jogoAcabou = false;
    this.jogadorGanhou = false;
}

CampoMinado.prototype = {
    quadrado: function (p) {
        return this.quadrados[p.linha][p.coluna];
    },

    posicaoEValida: function (p) {
        return 0 <= p.linha && p.linha < this.altura && 0 <= p.coluna && p.coluna < this.largura;
    },

    encontreAsPosicoesAoRedor: function (p) {
        var posicoes = [];
        //deslocamentos para as posições ao redor no sentido anti-horário
        var vizinhos = [[0, +1], [-1, +1], [-1, 0], [-1, -1], [0, -1], [+1, -1], [+1, 0], [+1, +1]];
        for (var i = 0; i < vizinhos.length; i += 1) {
            var np = new Posicao(p.linha + vizinhos[i][0], p.coluna + vizinhos[i][1]);
            if (this.posicaoEValida(np)) {
                posicoes.push(np);
            }
        }
        return posicoes;
    },

    mudeBandeira: function (p) {
        if (this.quadrado(p).estaAberto) {
            return;
        }
        if (this.quadrado(p).temBandeira) {
            this.quadrado(p).temBandeira = false
            this.numBandeiras -= 1;
        } else {
            this.quadrado(p).temBandeira = true;
            this.numBandeiras += 1;
        }
    },

    adicioneBomba: function (p) {
        if (this.quadrado(p).temBomba) {
            return;
        }
        this.quadrado(p).temBomba = true;
        this.numBombas += 1;
        this.aumenteNumBombasNosQuadradosAoRedor(p);
    },

    aumenteNumBombasNosQuadradosAoRedor: function(p) {
        var vizinhos = this.encontreAsPosicoesAoRedor(p);
        for (var i = 0; i < vizinhos.length; i += 1) {
            this.quadrado(vizinhos[i]).numBombasAoRedor += 1;
        }
    },

    sorteiUmaPosicao: function () {
        return new Posicao(randint(0, this.altura), randint(0, this.largura));
    },

    crieUmaBomba: function () {
        var p;
        do {
            p = this.sorteiUmaPosicao();
        } while (this.quadrado(p).temBomba);
        this.adicioneBomba(p)
    },

    crieBombas: function (numNovasBombas) {
        if (this.numQuadrados < this.numBombas + numNovasBombas) {
            throw "Não é possível criar mais bombas do que quadrados!";
        }
        while (numNovasBombas > 0) {
            this.crieUmaBomba();
            numNovasBombas -= 1;
        }
    },

    abraQuadrado: function (p) {
        if (! this.jogoAcabou && ! this.quadrado(p).estaAberto && ! this.quadrado(p).temBandeira) {
            if (this.quadrado(p).temBomba) {
                this.abraQuadradoComBomba(p);
            } else {
                this.abraQuadradoSemBomba(p);
            }
        }
    },

    abraQuadradoComBomba: function (p) {
        this.quadrado(p).estaAberto = true;
        this.numQuadradosAbertos += 1;
        this.jogoAcabou = true;
        this.jogadorGanhou = false;
    },

    abraQuadradoSemBomba: function (p) {
        this.quadrado(p).estaAberto = true;
        this.numQuadradosAbertos += 1;
        this.verifiqueJogadorGanhou();
        if (this.quadrado(p).numBombasAoRedor == 0) {
            this.abraQuadradosAoRedor(p);
        }
    },

    abraQuadradosAoRedor: function (p) {
        var vizinhos = this.encontreAsPosicoesAoRedor(p);
        for (var i = 0; i < vizinhos.length; i += 1) {
            this.abraQuadrado(vizinhos[i]);
        }
    },

    verifiqueJogadorGanhou: function () {
        if (this.numBombas + this.numQuadradosAbertos == this.numQuadrados) {
            this.jogoAcabou = true;
            this.jogadorGanhou = true;
        }
    }
}
